from datetime import datetime, timedelta, date

from django.core.urlresolvers import reverse_lazy, reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.views import generic

from ajax import vote_on_day
from models import Tag, Vote, VoteToTag
from patterns import DAY_SHORT_PATTERN, DAY_LONG_PATTERN


def get_day_context(day_string_short):
    # parse the incoming date and nicefy the string
    day_date = datetime.strptime(day_string_short, DAY_SHORT_PATTERN).date()
    day_string_long = day_date.strftime(DAY_LONG_PATTERN)

    # format day before and day after for links
    day_later_string_short = (day_date + timedelta(days=1)).strftime(DAY_SHORT_PATTERN)
    day_before_string_short = (day_date + timedelta(days=-1)).strftime(DAY_SHORT_PATTERN)

    # fetch possibly existing vote
    try:
        vote = Vote.objects.get(day=day_date)
    except Vote.DoesNotExist:
        vote = ''

    # create the different tag lists
    all_tags = Tag.objects.all()
    if vote:
        vote_tags = VoteToTag.objects.filter(vote=vote)
        positive_tags = map(lambda vote_tag: vote_tag.tag,
                            (filter(lambda vote_tag: vote_tag.is_positive, vote_tags)))
        negative_tags = map(lambda vote_tag: vote_tag.tag,
                            (filter(lambda vote_tag: not vote_tag.is_positive, vote_tags)))
    else:
        positive_tags = []
        negative_tags = []

    unused_tags = [item for item in all_tags if item not in (positive_tags + negative_tags)]

    context = {
        'day_later_string_short': day_later_string_short,
        'day_before_string_short': day_before_string_short,
        'day_string_short': day_string_short,
        'day_string_long': day_string_long,
        'vote': vote,
        'all_tags': all_tags,
        'unused_tags': unused_tags,
        'positive_tags': positive_tags,
        'negative_tags': negative_tags
    }

    return context


def home(request):
    today_date = date(datetime.now().year, datetime.now().month, datetime.now().day)
    day_context = get_day_context(today_date.strftime(DAY_SHORT_PATTERN))
    return render(request, 'RateYourLife/home.html', day_context)


def day(request, day_string_short):
    day_context = get_day_context(day_string_short)
    return render(request, 'RateYourLife/day.html', day_context)


def vote_on_mobile(request, day_string_short):
    try:
        selected_day = request.POST['hiddenDayString']
        positive_tags = request.POST.getlist('multiselectPositive[]')
        negative_tags = request.POST.getlist('multiselectNegative[]')
    except KeyError:
        return render(request, 'RateYourLife/error.html', {
            'error_message': "Something went wrong.",
        })

    # we can reuse vote_on_day for the voting part
    vote_on_day(request, selected_day)
    day_date = datetime.strptime(day_string_short, DAY_SHORT_PATTERN).date()
    vote = Vote.objects.get_or_create(day=day_date)[0]

    # we need to "hard reset" the tags
    for vote_to_tag in VoteToTag.objects.filter(vote=vote):
        vote_to_tag.delete()

    # now set them from the beginning
    for tag_name in positive_tags:
        tag = Tag.objects.get(name=tag_name)
        VoteToTag.objects.create(vote=vote, tag=tag, is_positive=True)
    for tag_name in negative_tags:
        tag = Tag.objects.get(name=tag_name)
        VoteToTag.objects.create(vote=vote, tag=tag, is_positive=False)

    return HttpResponseRedirect(reverse('RateYourLife:home'))


def day_mobile(request, day_string_short):
    day_context = get_day_context(day_string_short)
    return render(request, 'RateYourLife/day_mobile.html', day_context)


class TagList(generic.ListView):
    model = Tag


class TagDetail(generic.DetailView):
    model = Tag


class TagCreate(generic.CreateView):
    model = Tag
    success_url = reverse_lazy('RateYourLife:tag_list')


class TagUpdate(generic.UpdateView):
    model = Tag
    success_url = reverse_lazy('RateYourLife:tag_list')


class TagDelete(generic.DeleteView):
    model = Tag
    success_url = reverse_lazy('RateYourLife:tag_list')